from nilmdb import Interval, IntervalSet, IntervalError, FileInterval
from datetime import datetime
from nose.tools import assert_raises

from test_interval import iset

def fiset(string):
    """Like iset, but builds with FileIntervals instead of Intervals"""
    iset = IntervalSet()
    for i, c in enumerate(string):
        day = datetime.strptime("{0:04d}".format(i+2000), "%Y")
        if (c == "["):
            start = day
        elif (c == "|"):
            iset += FileInterval(start, day, "test.dat")
            start = day
        elif (c == "]"):
            iset += FileInterval(start, day, "test.dat")
            del start
    return iset

def test_fileinterval_vs_interval():
    """Test FileInterval/Interval inheritance"""

    i = iset("[--]")
    f = fiset("[--]")

    # check types
    assert(isinstance(i[0], Interval))
    assert(not isinstance(i[0], FileInterval))
    assert(isinstance(f[0], Interval))
    assert(isinstance(f[0], FileInterval))

    # when doing an intersection, result should be a subset of the first arg
    u = (i & f)
    assert(isinstance(u[0], Interval))
    assert(not isinstance(u[0], FileInterval))
    u = (f & i)
    assert(isinstance(u[0], Interval))
    assert(isinstance(u[0], FileInterval))

    # they're still the same though
    assert(i == f == u)

    # just for coverage
    assert_raises(IntervalError, fiset("[]")[0].subset, f[0].start, f[0].end)
